﻿using ELTE.TravelAgency.Data;
using ELTE.TravelAgency.Service.Models;
using System;
using System.Linq;
using System.Web.Http;

namespace ELTE.TravelAgency.Service.Controllers
{
    /// <summary>
    /// Épületek lekérdezését és módosítását biztosító vezérlő.
    /// </summary>
    public class BuildingsController : ApiController
    {
        private ITravelAgencyEntities _entities;

        /// <summary>
        /// Vezérlő példányosítása.
        /// </summary>
        public BuildingsController()
            : this(new TravelAgencyEntities())
        {
        }
        
        /// <summary>
        /// Vezérlő példányosítása.
        /// </summary>
        /// <param name="entities">Entitásmodell.</param>
        public BuildingsController(ITravelAgencyEntities entities)
        {
            if (entities == null)
                throw new ArgumentNullException("entities");

            _entities = entities;
        }
        
        /// <summary>
        /// Vezérlő megsemmisítése.
        /// </summary>
        ~BuildingsController()
        {
            Dispose(false);
        }

        /// <summary>
        /// Épületek lekérdezése.
        /// </summary>
        public IHttpActionResult GetBuildings()
        {
            try
            {
                return Ok(_entities.Building.ToList().Select(building => new BuildingDTO
                {
                    Id = building.Id,
                    Name = building.Name,
                    CityId = building.CityId,
                    SeaDistance = building.SeaDistance,
                    Features = building.Features,
                    ShoreId = building.ShoreId,
                    LocationX = building.LocationX,
                    LocationY = building.LocationY,
                    Comment = building.Comment
                }));
            }
            catch
            {
                return InternalServerError();
            }
        }

        /// <summary>
        /// Épületek lekérdezése.
        /// </summary>
        /// <param name="id">Város azonosító.</param>
        public IHttpActionResult GetBuildingForCity(Int32 id)
        {
            try
            {
                return Ok(_entities.Building.Where(b => b.CityId == id).ToList().Select(building => new BuildingDTO
                {
                    Id = building.Id,
                    Name = building.Name,
                    CityId = building.CityId,
                    SeaDistance = building.SeaDistance,
                    Features = building.Features,
                    ShoreId = building.ShoreId,
                    LocationX = building.LocationX,
                    LocationY = building.LocationY,
                    Comment = building.Comment
                }));
            }
            catch
            {
                return InternalServerError();
            }
        }

        /// <summary>
        /// Új épület létrehozása.
        /// </summary>
        /// <param name="buildingDTO">Épület.</param>
        public IHttpActionResult PostBuilding([FromBody] BuildingDTO buildingDTO)
        {
            try
            {
                Building addedBuilding = _entities.Building.Add(new Building
                {
                    Name = buildingDTO.Name,
                    CityId = buildingDTO.CityId,
                    SeaDistance = buildingDTO.SeaDistance,
                    Features = buildingDTO.Features,
                    ShoreId = buildingDTO.ShoreId,
                    LocationX = buildingDTO.LocationX,
                    LocationY = buildingDTO.LocationY,
                    Comment = buildingDTO.Comment
                });

                _entities.SaveChanges(); // elmentjük az új épületet

                buildingDTO.Id = buildingDTO.Id;

                // visszaküldjük a létrehozott épületet
                return Created(Request.RequestUri + addedBuilding.Id.ToString(), buildingDTO);
            }
            catch
            {
                return InternalServerError();
            }
        }

        /// <summary>
        /// Épület módosítása.
        /// </summary>
        /// <param name="buildingDTO">Épület.</param>
        public IHttpActionResult PutBuilding([FromBody] BuildingDTO buildingDTO)
        {
            try
            {
                Building building = _entities.Building.FirstOrDefault(b => b.Id == buildingDTO.Id);

                if (building == null) // ha nincs ilyen azonosító, akkor hibajelzést küldünk
                    return NotFound();

                building.Name = buildingDTO.Name;
                building.CityId = buildingDTO.CityId;
                building.SeaDistance = buildingDTO.SeaDistance;
                building.Features = buildingDTO.Features;
                building.ShoreId = buildingDTO.ShoreId;
                building.LocationX = buildingDTO.LocationX;
                building.LocationY = buildingDTO.LocationY;
                building.Comment = buildingDTO.Comment;

                _entities.SaveChanges(); // elmentjük a módosított épületet

                return Ok();
            }
            catch
            {
                return InternalServerError();
            }
        }

        /// <summary>
        /// Épület törlése.
        /// </summary>
        /// <param name="id">Épület azonosító.</param>
        public IHttpActionResult DeleteBuilding(Int32 id)
        {
            try
            {
                Building building = _entities.Building.FirstOrDefault(b => b.Id == id);

                if (building == null) // ha nincs ilyen azonosító, akkor hibajelzést küldünk
                    return NotFound();

                _entities.Building.Remove(building);

                _entities.SaveChanges(); // elmentjük a módosított épületet

                return Ok();
            }
            catch
            {
                return InternalServerError();
            }
        }

        /// <summary>
        /// Megsemmisítés.
        /// </summary>
        protected override void Dispose(Boolean disposing)
        {
            if (disposing)
            {
                _entities.Dispose();
            }

            base.Dispose(disposing);
        }
    }
}
